#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "RandomGenerator.h"

namespace TLUtil
{

  void RandomGenerator::seed()
  {
    time_t ltime;
    struct tm *today;
    time(&ltime);
    today = localtime(&ltime);
    manualSeed((unsigned long)today->tm_sec);
    //manualSeed(5489UL);  // changed temporarily
  }

///////////// The next 4 methods are taken from http://www.math.keio.ac.jp/matumoto/emt.html
///////////// Here is the copyright:
///////////// Some minor modifications have been made to adapt to "my" C++...

/*
   A C-program for MT19937, with initialization improved 2002/2/10.
   Coded by Takuji Nishimura and Makoto Matsumoto.
   This is a faster version by taking Shawn Cokus's optimization,
   Matthe Bellew's simplification, Isaku Wada's float version.

   Before using, initialize the state by using init_genrand(seed)
   or init_by_array(init_key, key_length).

   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote
        products derived from this software without specific prior written
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


   Any feedback is very welcome.
   http://www.math.keio.ac.jp/matumoto/emt.html
   email: matumoto@math.keio.ac.jp
*/

////////////////// Macros for the Mersenne Twister random generator...
/* Period parameters */  
//#define n 624
//#define m 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UMASK 0x80000000UL /* most significant w-r bits */
#define LMASK 0x7fffffffUL /* least significant r bits */
#define MIXBITS(u,v) ( ((u) & UMASK) | ((v) & LMASK) )
#define TWIST(u,v) ((MIXBITS(u,v) >> 1) ^ ((v)&1UL ? MATRIX_A : 0UL))
/////////////////////////////////////////////////////////// That's it.

  RandomGenerator::RandomGenerator(){
    ///// Code for the Mersenne Twister random generator....
    left = 1;
    initf = 0;
    state = new unsigned long [n]; /* the array for the state vector  */
    normal_is_valid = false;
    seed();
  }

  RandomGenerator::RandomGenerator(unsigned long s){
    ///// Code for the Mersenne Twister random generator....
    left = 1;
    initf = 0;
    state = new unsigned long [n]; /* the array for the state vector  */
    normal_is_valid = false;
    manualSeed(s);
  }


  void RandomGenerator::manualSeed(unsigned long s)
  {
    state[0]= s & 0xffffffffUL;
    for(int j = 1; j < n; j++)
      {
	state[j] = (1812433253UL * (state[j-1] ^ (state[j-1] >> 30)) + j); 
	/* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
	/* In the previous versions, mSBs of the seed affect   */
	/* only mSBs of the array state[].                        */
	/* 2002/01/09 modified by makoto matsumoto             */
	state[j] &= 0xffffffffUL;  /* for >32 bit machines */
      }
    left = 1;
    initf = 1;
  }
  
  
  void RandomGenerator::nextState()
  {
    unsigned long *p=state;
    
    /* if init_genrand() has not been called, */
    /* a default initial seed is used         */
    if(initf == 0)
      seed();
    //    manualSeed(5489UL);
    
    left = n;
    next = state;
    
    for(int j = n-m+1; --j; p++) 
      *p = p[m] ^ TWIST(p[0], p[1]);

    for(int j = m; --j; p++) 
      *p = p[m-n] ^ TWIST(p[0], p[1]);

    *p = p[m-n] ^ TWIST(p[0], state[0]);
  }

  unsigned long RandomGenerator::random()
  {
    unsigned long y;
    
    if (--left == 0)
      nextState();
    y = *next++;
  
    /* Tempering */
    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);
    
    return y;
  }

  /* generates a random number on [0,1)-float-interval */
  float RandomGenerator::uniform()
  {
    unsigned long y;
    
    if(--left == 0)
      nextState();
    y = *next++;
    
    /* Tempering */
    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);
    
    //return (float)y * (1.0/4294967296.0);   // it seems the result can be 1.0 which I want to avoid
    return (float)y * (1.0/4294967300.0); 
    /* divided by 2^32 */
  }
  
  ///
  /// Thanks *a lot* Takuji Nishimura and Makoto Matsumoto!
  ///
  /////////////////////////////////////////////////////////////////////
  //// Now my own code...

  void RandomGenerator::getShuffledIndices(int *indices, int n_indices)
  {
    for(int i = 0; i < n_indices; i++)
      indices[i] = i;
    
    shuffle(indices, sizeof(int), n_indices);
  }

  void RandomGenerator::shuffle(void *tabular, int size_elem, int n_elems)
  {
    void *save = malloc(size_elem);
    char *tab = (char *)tabular;
    
    for(int i = 0; i < n_elems; i++)
      {
	int z = (int)(uniform() * ((float)(n_elems-i-1)) );
	memcpy(save, tab+i*size_elem, size_elem);
	memcpy(tab+i*size_elem, tab+(z+i)*size_elem, size_elem);
	memcpy(tab+(z+i)*size_elem, save, size_elem);
      }
    free(save);
  }
  
  float RandomGenerator::boundedUniform(float a, float b)
  {
    return(uniform() * (b - a) + a);
  }
  
  float RandomGenerator::normal(float mean, float stdv)
  {
    if(!normal_is_valid)
      {
	normal_x = uniform();
	normal_y = uniform();
	normal_rho = sqrt(-2. * log(1.0-normal_y));
	normal_is_valid = true;
      }
    else
      normal_is_valid = false;
    
    if(normal_is_valid)
      return normal_rho*cos(2.*M_PI*normal_x)*stdv+mean;
    else
      return normal_rho*sin(2.*M_PI*normal_x)*stdv+mean;
  }

  float RandomGenerator::exponential(float lambda)
  {
    return(-1. / lambda * log(1-uniform()));
  }

  float RandomGenerator::cauchy(float median, float sigma)
  {
    return(median + sigma * tan(M_PI*(uniform()-0.5)));
  }
  
  // Faut etre malade pour utiliser ca.
  // M'enfin.
  float RandomGenerator::logNormal(float mean, float stdv)
  {
    float zm = mean*mean;
    float zs = stdv*stdv;
    return(exp(normal(log(zm/sqrt(zs + zm)), sqrt(log(zs/zm+1)) )));
  }
  
  int RandomGenerator::geometric(float p)
  {
    return((int)(log(1-uniform()) / log(p)) + 1);
  }

  bool RandomGenerator::bernouilli(float p)
  {
    return(RandomGenerator::uniform() <= p);
  }

  int RandomGenerator::fromHistogram(float* cum_hist, int nb_values)
  {
    float uni = boundedUniform(0, cum_hist[nb_values-1]);
    int i;

    i=0;
    while (i<nb_values && uni>cum_hist[i])
      i++;
    if (i>=nb_values)
      return nb_values-1;
    else
      return i;
  }

  RandomGenerator::~RandomGenerator(){
    delete [] state;
  }

}


